%
%
%  Detect Spots in all Channels for Imaris 7.6.5
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Detect Spots in all  Channels" icon="Matlab" tooltip="Detect Spots in all Channels">
%          <Command>MatlabXT::XTDetectSpotsAllChannels(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%  
%  Description:
%
%   This program is designed to be used with the XTBatchProcess xtension to detect and
%   create spots objects in all channels based on user parameters for a folder of .ims images.
%
%

function XTDetectSpotsAllChannels(aImarisApplicationID)

    %Connect to Imaris based on Imaris 7.6 XT Interface documentation
    if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
        vImarisApp = aImarisApplicationID;
    else
        if exist('ImarisLib','class') == 0
            javaaddpath ImarisLib.jar
        end
        vImarisLib = ImarisLib;
        aServer = vImarisLib.GetServer;
        numObjects = aServer.GetNumberOfObjects;
        for vIndex = 0:numObjects-1
            aObjectId = aServer.GetObjectID(vIndex);
            break
        end
        if ischar(aObjectId)
            aObjectId = round(aObjectId);
        end
        vImarisApp = vImarisLib.GetApplication(aObjectId);
    end

    %Check for connection to Imaris
    if(isequal(vImarisApp, []))
        display('Could not connect to Imaris')
        return 
    end

    %Get image
    vImage = vImarisApp.GetDataSet();
    if(isequal(vImage, []))
        display('An image must be loaded for this XTension')
        return
    end

    %Declare global variables
    global estDiameter subBackground autoThreshold threshold localContrast regThresholdAuto regThresholdMan aRegionsSpotsDiameterFromVolume aRegionsCreateChannel
    
    currentFile = char(vImarisApp.GetCurrentFileName());
    channelCount = vImage.GetSizeC(); 
    scene = vImarisApp.GetSurpassScene();
   
    initializeParameters();
    ROI = int16.empty(8,0); %empty ROI = include entire image
    filterString = '"Quality" above '; %"Quality" above threshold given as following parameter

    for c=1:(channelCount) %For each channel
    %SETTINGS TO DETECT SPOTS 
       getSpotCreationParametersFromUser();
       channelIndex = c-1;
       if autoThreshold
           fullFilterString = [filterString 'automatic threshold'];
       else
            fullFilterString = [filterString num2str(threshold)];
       end
       spots = vImarisApp.GetImageProcessing.DetectSpotsRegionGrowing(vImage, ROI, channelIndex, estDiameter, subBackground, fullFilterString, localContrast, regThresholdAuto, regThresholdMan, aRegionsSpotsDiameterFromVolume, aRegionsCreateChannel);
       scene.AddChild(spots, -1);
    end
    channelOutputImage = strcat(currentFile, '_withSpots.ims');
    vImarisApp.FileSave(channelOutputImage, 'writer="Imaris5"');
end

function getSpotCreationParametersFromUser()
    global estDiameter subBackground autoThreshold threshold localContrast regThresholdAuto regThresholdMan aRegionsSpotsDiameterFromVolume aRegionsCreateChannel
    
    prompt={'Estimated Diameter', 'Background Subtraction (0/1)', 'Automatic Quality Threshold (0/1)', 'Quality Threshold Value','Local Contrast (0/1)', 'Use Auto Region Threshold (0/1)', 'Manual Regional Threshold Value','Region Spots Diameter from Volume (0/1)', 'Create Region Channel (0/1)'};
    defaults = {num2str(estDiameter), num2str(subBackground), num2str(autoThreshold),num2str(threshold), num2str(localContrast), num2str(regThresholdAuto), num2str(regThresholdMan), num2str(aRegionsSpotsDiameterFromVolume),num2str(aRegionsCreateChannel)};
    userInput=inputdlg(prompt, strcat('Spot Creation Parameters'), 1, defaults);
    estDiameter = str2double(userInput{1}); %diameter restrictions
    subBackground = str2double(userInput{2}); %boolean for subtracting the background
    autoThreshold = str2double(userInput{3}); %boolean, 0 means use the threshold values given, 1 means ignore the given quality threshold and use the autothreshold
    threshold = str2double(userInput{4}); %used in filterString
    localContrast = str2double(userInput{5}); %boolean
    regThresholdAuto = str2double(userInput{6}); %boolean, 0 means use the regThresholdMan values given as a parameter, 1 means ignore the regThresholdMan and use AutoThresholding
    regThresholdMan = str2double(userInput{7}); %not used if regThresholdAuto = 1
    aRegionsSpotsDiameterFromVolume = str2double(userInput{8});
    aRegionsCreateChannel = str2double(userInput{9});
    validateUserInput();
end

function validateUserInput()
    global estDiameter subBackground autoThreshold threshold localContrast regThresholdAuto regThresholdMan aRegionsSpotsDiameterFromVolume aRegionsCreateChannel
    
    if ~(subBackground == 0 || subBackground == 1)
        waitfor(errordlg('Background Subtraction must be 0 for false or 1 for true'));
        subBackground = 1;
    elseif ~(autoThreshold == 0 || autoThreshold == 1)
        waitfor(errordlg('Quality Auto Threshold must be 0 for false or 1 for true'));
        autoThreshold = 1;
    elseif ~(localContrast == 0 || localContrast == 1)
        waitfor(errordlg('Local Contrast must be 0 for false or 1 for true'));
        localContrast = 0;
    elseif ~(regThresholdAuto == 0 || regThresholdAuto == 1)
        waitfor(errordlg('Auto Regional Threshold must be 0 for false or 1 for true'));
        regThresholdAuto = 0;
    elseif ~(aRegionsSpotsDiameterFromVolume == 0 || aRegionsSpotsDiameterFromVolume == 1)
            waitfor(errordlg('Auto Regional Threshold must be 0 for false or 1 for true'));
        aRegionsSpotsDiameterFromVolume = 0;
    elseif ~(aRegionsCreateChannel == 0 || aRegionsCreateChannel == 1)
            waitfor(errordlg('Region Create Channel must be 0 for false or 1 for true'));
        aRegionsCreateChannel = 0;
    elseif ~(estDiameter >= 0)
        waitfor(errordlg('The estimated diameter must be greater than or equal to 0'));
        estDiameter = 0.5;
    elseif ~(threshold >= 0)
        waitfor(errordlg('The Quality threshold must be greater than or equal to 0'));
        threshold = 10;
    elseif ~(regThresholdMan >= 0)
        waitfor(errordlg('The manual regional threshold must be greater than or equal to 0'));
        regThresholdMan = 10; 
    else
        return; %all the input was valid - return
    end
    getSpotCreationParametersFromUser(); % not all of the input was valid, ask the user for input again
end

function initializeParameters()
    global estDiameter subBackground autoThreshold threshold localContrast regThresholdAuto regThresholdMan aRegionsSpotsDiameterFromVolume aRegionsCreateChannel
    
    estDiameter = 0.5; %diameter restrictions
    subBackground = 1; %boolean for subtracting the background
    autoThreshold = 1;
    threshold = 20;
    localContrast = 0; %boolean
    regThresholdAuto = 0; %boolean, 0 means use the regThresholdMan values given as a parameter, 1 means ignore the regThresholdMan and use AutoThresholding
    regThresholdMan = 20; %not used if regThresholdAuto = 1
    aRegionsSpotsDiameterFromVolume = 0;
    aRegionsCreateChannel = 0;
end